import dbus
from lxml import etree
import os

class pyrbox:

    libraryloaded = False
    bus = None
    player = None
    shell = None
    plmanager = None
    xmldoc = None

    def __init__(self):
        self.bus = dbus.SessionBus()
        #get the player and shell objects
        pobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Player')
        pifacename = "org.gnome.Rhythmbox.Player"
        shellobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Shell')
        shellifacename = "org.gnome.Rhythmbox.Shell"
        pmanobj = self.bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/PlaylistManager')
        pmanifacename = "org.gnome.Rhythmbox.PlaylistManager"
        #get the interface 2 access methods,properties,etc
        self.player = dbus.Interface(pobj, pifacename)
        self.shell = dbus.Interface(shellobj, shellifacename)
        self.plmanager = dbus.Interface(pmanobj, pmanifacename)

    def loadLibraryFile(self):
        # check to make sure a library file exist
        if os.path.exists("rboxMusicLib.xml"):
            # load music library file
            self.xmldoc = etree.parse("rboxMusicLib.xml")
            self.libraryloaded = True
            return True
        return False

    def getPlayingSongProps(self):
        return self.shell.getSongProperties(self.player.getPlayingUri())

    def uriByTrack(self, *args):
        if self.libraryloaded==False: return False
        bFileOpt = False # flag for if filename was passed(3rd arg)
        if len(args)==2: #artist=args[0], title=args[1]
            if args[0]=="" or args[1]=="": return False
        elif len(args)==3:
            if args[0]=="" or args[1]=="" or args[2]=="": return False
            bFileOpt = True

        # run the xpath query depending on # of args
        if bFileOpt == False:
            query = "//track[translate(title, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'," \
                + "'abcdefghijklmnopqrstuvwxyz')='"+args[1].lower() \
                +"' and translate(artist, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', " \
                + "'abcdefghijklmnopqrstuvwxyz')='"+ args[0].lower() + "']/./uri/text()"
            res = self.xmldoc.xpath(query)
            return res
        else:
            query = "//track[translate(title, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'," \
                + "'abcdefghijklmnopqrstuvwxyz')='"+args[1].lower() \
                +"' and translate(artist, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', " \
                + "'abcdefghijklmnopqrstuvwxyz')='"+ args[0].lower() + "']/./uri/text()"
            res = self.xmldoc.xpath(query)
            # because the result in always a dict object (an array) this (below) 
            #  grabs the 1st entry (which there should only be one unless you have
            #  multiple tracks with the exact same filename/artistname/trackname) 
            #  and returns it as just a string (so minus the '[' ']')
            if len(res) > 0:
                for i in range(len(res)):
                    if self.filenameFromUri(res[i]).lower() == args[2].lower():
                        return res[i]
        return False

    def getUri(self, tagname, value):
        if self.libraryloaded==False or tagname=="" or value=="": return ""
        # use xpaths translate function to convert to lowercase
        res = self.xmldoc.xpath("//track[translate("+tagname
            + ", 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')='"
            + value.lower()+"']/./uri/text()")
        return res

    def uriByAlbum(self, albumname):
        if self.libraryloaded==False or trackname=="": return ""
        return self.getUri("album", albumname)

    def uriByTitle(self, trackname):
        if self.libraryloaded==False or trackname=="": return ""
        return self.getUri("title", trackname)

    def uriByArtist(self, artistname):
        if self.libraryloaded==False or artistname=="": return ""
        return self.getUri("artist", artistname)

    def tracksByTitle(self, trackname):
        if self.libraryloaded==False or trackname=="": return ""
        # return list of tracks
        res = self.xmldoc.xpath("//track[translate(title,'ABCDEFGHIJKLMNOPQRSTUVWXYZ',"
        + "'abcdefghijklmnopqrstuvwxyz')='"+trackname.lower()+"']")
        # the result will be: res[n][0]=uri, [n][1]=artist, [2]title,
        #  [3]album, [4]genre
        return res

    def tracksByArtist(self, artistname):
        if self.libraryloaded==False or artistname=="": return ""
        # return list of tracks
        res = self.xmldoc.xpath("//track[translate(artist,'ABCDEFGHIJKLMNOPQRSTUVWXYZ',"
            + "'abcdefghijklmnopqrstuvwxyz')='"+artistname.lower()+"']")
        # the result will be: res[n][0]=uri, [n][1]=artist, [2]title,
        #  [3]album, [4]genre
        return res

    def tracksByAlbum(self, albumname):
        if self.libraryloaded==False or albumname=="": return ""
        # return list of tracks
        res = self.xmldoc.xpath("//track[album='"+ albumname +"']")
        # the result will be: res[n][0]=uri, [n][1]=artist, [2]title,
        #  [3]album, [4]genre
        return res

    def filenameFromUri(self, uri):
        pos = uri.rfind("/")
        filename = uri[pos+1:]
        return filename

    def add2playlist(self, plistname, trackuri):
        if plistname=="" or trackuri=="": return False
        # add track to playlist
        if self.plmanager.addToPlaylist(plistname, trackuri)==False: return False
        return True

    def createPlaylist(self, plistname):
        if plistname == "": return False
        res = self.plmanager.createPlaylist(plistname)
        if res == False: return False
        return True
